import pytest
from elevenlabs.client import AsyncElevenLabs, ElevenLabs

from .utils import DEFAULT_VOICE_FILE

DEFAULT_EXT_AUDIO = "https://storage.googleapis.com/eleven-public-cdn/audio/marketing/nicole.mp3"



@pytest.mark.asyncio
async def test_stt_convert():
    """Test basic speech-to-text conversion."""
    client = ElevenLabs()

    audio_file = open(DEFAULT_VOICE_FILE, "rb")

    transcription = client.speech_to_text.convert(
        file=audio_file,
        model_id="scribe_v1"
    )

    assert isinstance(transcription.text, str)
    assert len(transcription.text) > 0
    assert isinstance(transcription.words, list)
    assert len(transcription.words) > 0



"""Tests for SpeechEngineClient and AsyncSpeechEngineClient wrapper methods."""

from unittest.mock import AsyncMock, MagicMock, patch

import pytest

from elevenlabs.speech_engine.client import AsyncSpeechEngineClient as AutogeneratedAsyncSpeechEngineClient
from elevenlabs.speech_engine.client import SpeechEngineClient as AutogeneratedSpeechEngineClient
from elevenlabs.speech_engine.resource import SpeechEngineResource
from elevenlabs.speech_engine_custom import AsyncSpeechEngineClient, SpeechEngineClient
from elevenlabs.types.speech_engine_config import SpeechEngineConfig
from elevenlabs.types.speech_engine_response import SpeechEngineResponse


def _make_sync_client() -> SpeechEngineClient:
    return SpeechEngineClient(client_wrapper=MagicMock())


def _make_async_client() -> AsyncSpeechEngineClient:
    return AsyncSpeechEngineClient(client_wrapper=MagicMock())


# ---------------------------------------------------------------------------
# Sync
# ---------------------------------------------------------------------------


def test_create_returns_resource() -> None:
    client = _make_sync_client()
    mock_response = MagicMock(spec=SpeechEngineResponse, speech_engine_id="seng_abc")

    with patch.object(AutogeneratedSpeechEngineClient, "create", return_value=mock_response):
        result = client.create(speech_engine=SpeechEngineConfig(ws_url="wss://test"))

    assert isinstance(result, SpeechEngineResource)
    assert result.engine_id == "seng_abc"


def test_get_returns_resource() -> None:
    client = _make_sync_client()

    with patch.object(AutogeneratedSpeechEngineClient, "get", return_value=None):
        result = client.get("seng_abc")

    assert isinstance(result, SpeechEngineResource)
    assert result.engine_id == "seng_abc"


def test_update_returns_resource() -> None:
    client = _make_sync_client()

    with patch.object(AutogeneratedSpeechEngineClient, "update", return_value=None):
        result = client.update("seng_abc", name="Renamed")

    assert isinstance(result, SpeechEngineResource)
    assert result.engine_id == "seng_abc"


# ---------------------------------------------------------------------------
# Async
# ---------------------------------------------------------------------------


@pytest.mark.asyncio
async def test_async_create_returns_resource() -> None:
    client = _make_async_client()
    mock_response = MagicMock(spec=SpeechEngineResponse, speech_engine_id="seng_abc")

    with patch.object(AutogeneratedAsyncSpeechEngineClient, "create", new_callable=AsyncMock, return_value=mock_response):
        result = await client.create(speech_engine=SpeechEngineConfig(ws_url="wss://test"))

    assert isinstance(result, SpeechEngineResource)
    assert result.engine_id == "seng_abc"


@pytest.mark.asyncio
async def test_async_get_returns_resource() -> None:
    client = _make_async_client()

    with patch.object(AutogeneratedAsyncSpeechEngineClient, "get", new_callable=AsyncMock, return_value=None):
        result = await client.get("seng_abc")

    assert isinstance(result, SpeechEngineResource)
    assert result.engine_id == "seng_abc"


@pytest.mark.asyncio
async def test_async_update_returns_resource() -> None:
    client = _make_async_client()

    with patch.object(AutogeneratedAsyncSpeechEngineClient, "update", new_callable=AsyncMock, return_value=None):
        result = await client.update("seng_abc", name="Renamed")

    assert isinstance(result, SpeechEngineResource)
    assert result.engine_id == "seng_abc"



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class UpdateOAuth2ClientCredsRequest(UncheckedBaseModel):
    provider: typing.Optional[str] = None
    client_id: typing.Optional[str] = None
    scopes: typing.Optional[typing.List[str]] = None
    extra_params: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None
    basic_auth_in_header: typing.Optional[bool] = None
    client_secret: typing.Optional[str] = None
    custom_headers: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel, UnionMetadata
from .api_integration_o_auth_2_auth_code_response_scope_separator import (
    ApiIntegrationOAuth2AuthCodeResponseScopeSeparator,
)
from .api_integration_o_auth_2_custom_app_response_scope_separator import (
    ApiIntegrationOAuth2CustomAppResponseScopeSeparator,
)
from .auth_connection_dependencies import AuthConnectionDependencies
from .o_auth_2_jwt_response_algorithm import OAuth2JwtResponseAlgorithm
from .o_auth_2_jwt_response_token_response_field import OAuth2JwtResponseTokenResponseField
from .o_auth_connection_status import OAuthConnectionStatus
from .private_key_jwt_response_algorithm import PrivateKeyJwtResponseAlgorithm


class ListAuthConnectionsResponseAuthConnectionsItem_ApiIntegrationOauth2AuthCode(UncheckedBaseModel):
    """
    The type of auth connection config
    """

    auth_type: typing.Literal["api_integration_oauth2_auth_code"] = "api_integration_oauth2_auth_code"
    name: str
    provider: str
    token_url: str
    scopes: typing.Optional[typing.List[str]] = None
    scope_separator: typing.Optional[ApiIntegrationOAuth2AuthCodeResponseScopeSeparator] = None
    expires_at: str
    integration_id: str
    credential_id: str
    status: typing.Optional[OAuthConnectionStatus] = None
    status_detail: typing.Optional[str] = None
    status_updated_at: typing.Optional[str] = None
    id: str
    used_by: typing.Optional[AuthConnectionDependencies] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ListAuthConnectionsResponseAuthConnectionsItem_ApiIntegrationOauth2CustomApp(UncheckedBaseModel):
    """
    The type of auth connection config
    """

    auth_type: typing.Literal["api_integration_oauth2_custom_app"] = "api_integration_oauth2_custom_app"
    name: str
    provider: str
    token_url: str
    scopes: typing.Optional[typing.List[str]] = None
    scope_separator: typing.Optional[ApiIntegrationOAuth2CustomAppResponseScopeSeparator] = None
    expires_at: str
    integration_id: str
    credential_id: str
    status: typing.Optional[OAuthConnectionStatus] = None
    status_detail: typing.Optional[str] = None
    status_updated_at: typing.Optional[str] = None
    client_id: str
    id: str
    used_by: typing.Optional[AuthConnectionDependencies] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ListAuthConnectionsResponseAuthConnectionsItem_BasicAuth(UncheckedBaseModel):
    """
    The type of auth connection config
    """

    auth_type: typing.Literal["basic_auth"] = "basic_auth"
    name: str
    provider: str
    username: str
    id: str
    used_by: typing.Optional[AuthConnectionDependencies] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ListAuthConnectionsResponseAuthConnectionsItem_BearerAuth(UncheckedBaseModel):
    """
    The type of auth connection config
    """

    auth_type: typing.Literal["bearer_auth"] = "bearer_auth"
    name: str
    provider: str
    id: str
    used_by: typing.Optional[AuthConnectionDependencies] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ListAuthConnectionsResponseAuthConnectionsItem_CustomHeaderAuth(UncheckedBaseModel):
    """
    The type of auth connection config
    """

    auth_type: typing.Literal["custom_header_auth"] = "custom_header_auth"
    name: str
    provider: str
    header_name: str
    id: str
    used_by: typing.Optional[AuthConnectionDependencies] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ListAuthConnectionsResponseAuthConnectionsItem_Mtls(UncheckedBaseModel):
    """
    The type of auth connection config
    """

    auth_type: typing.Literal["mtls"] = "mtls"
    name: str
    provider: str
    id: str
    used_by: typing.Optional[AuthConnectionDependencies] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ListAuthConnectionsResponseAuthConnectionsItem_Oauth2ClientCredentials(UncheckedBaseModel):
    """
    The type of auth connection config
    """

    auth_type: typing.Literal["oauth2_client_credentials"] = "oauth2_client_credentials"
    name: str
    provider: str
    client_id: str
    token_url: str
    scopes: typing.Optional[typing.List[str]] = None
    extra_params: typing.Optional[typing.Dict[str, str]] = None
    basic_auth_in_header: typing.Optional[bool] = None
    id: str
    used_by: typing.Optional[AuthConnectionDependencies] = None
    custom_headers: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ListAuthConnectionsResponseAuthConnectionsItem_Oauth2Jwt(UncheckedBaseModel):
    """
    The type of auth connection config
    """

    auth_type: typing.Literal["oauth2_jwt"] = "oauth2_jwt"
    name: str
    provider: str
    algorithm: typing.Optional[OAuth2JwtResponseAlgorithm] = None
    key_id: typing.Optional[str] = None
    issuer: str
    audience: str
    subject: str
    expiration_seconds: typing.Optional[int] = None
    extra_params: typing.Optional[typing.Dict[str, str]] = None
    token_url: str
    scopes: typing.Optional[typing.List[str]] = None
    token_response_field: typing.Optional[OAuth2JwtResponseTokenResponseField] = None
    id: str
    used_by: typing.Optional[AuthConnectionDependencies] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ListAuthConnectionsResponseAuthConnectionsItem_PrivateKeyJwt(UncheckedBaseModel):
    """
    The type of auth connection config
    """

    auth_type: typing.Literal["private_key_jwt"] = "private_key_jwt"
    name: str
    provider: str
    algorithm: typing.Optional[PrivateKeyJwtResponseAlgorithm] = None
    key_id: typing.Optional[str] = None
    issuer: str
    audience: str
    subject: str
    expiration_seconds: typing.Optional[int] = None
    extra_params: typing.Optional[typing.Dict[str, str]] = None
    id: str
    used_by: typing.Optional[AuthConnectionDependencies] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ListAuthConnectionsResponseAuthConnectionsItem_SlackBotAuth(UncheckedBaseModel):
    """
    The type of auth connection config
    """

    auth_type: typing.Literal["slack_bot_auth"] = "slack_bot_auth"
    name: str
    provider: typing.Optional[typing.Literal["Slack"]] = None
    id: str
    used_by: typing.Optional[AuthConnectionDependencies] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ListAuthConnectionsResponseAuthConnectionsItem_UrlSecret(UncheckedBaseModel):
    """
    The type of auth connection config
    """

    auth_type: typing.Literal["url_secret"] = "url_secret"
    name: str
    provider: str
    id: str
    used_by: typing.Optional[AuthConnectionDependencies] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ListAuthConnectionsResponseAuthConnectionsItem_WhatsappAuth(UncheckedBaseModel):
    """
    The type of auth connection config
    """

    auth_type: typing.Literal["whatsapp_auth"] = "whatsapp_auth"
    name: str
    provider: typing.Optional[typing.Literal["whatsapp"]] = None
    phone_number_id: str
    id: str
    used_by: typing.Optional[AuthConnectionDependencies] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ListAuthConnectionsResponseAuthConnectionsItem = typing_extensions.Annotated[
    typing.Union[
        ListAuthConnectionsResponseAuthConnectionsItem_ApiIntegrationOauth2AuthCode,
        ListAuthConnectionsResponseAuthConnectionsItem_ApiIntegrationOauth2CustomApp,
        ListAuthConnectionsResponseAuthConnectionsItem_BasicAuth,
        ListAuthConnectionsResponseAuthConnectionsItem_BearerAuth,
        ListAuthConnectionsResponseAuthConnectionsItem_CustomHeaderAuth,
        ListAuthConnectionsResponseAuthConnectionsItem_Mtls,
        ListAuthConnectionsResponseAuthConnectionsItem_Oauth2ClientCredentials,
        ListAuthConnectionsResponseAuthConnectionsItem_Oauth2Jwt,
        ListAuthConnectionsResponseAuthConnectionsItem_PrivateKeyJwt,
        ListAuthConnectionsResponseAuthConnectionsItem_SlackBotAuth,
        ListAuthConnectionsResponseAuthConnectionsItem_UrlSecret,
        ListAuthConnectionsResponseAuthConnectionsItem_WhatsappAuth,
    ],
    UnionMetadata(discriminant="auth_type"),
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.add_workspace_invite_response_model import AddWorkspaceInviteResponseModel
from ...types.delete_workspace_invite_response_model import DeleteWorkspaceInviteResponseModel
from ...types.seat_type import SeatType
from .raw_client import AsyncRawInvitesClient, RawInvitesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class InvitesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawInvitesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawInvitesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawInvitesClient
        """
        return self._raw_client

    def create(
        self,
        *,
        email: str,
        workspace_permission: typing.Optional[str] = OMIT,
        seat_type: typing.Optional[SeatType] = OMIT,
        group_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddWorkspaceInviteResponseModel:
        """
        Sends an email invitation to join your workspace to the provided email. If the user doesn't have an account they will be prompted to create one. If the user accepts this invite they will be added as a user to your workspace and your subscription using one of your seats. This endpoint may only be called by workspace members with the WORKSPACE_MEMBERS_INVITE permission. If the user is already in the workspace a 400 error will be returned.

        Parameters
        ----------
        email : str
            The email of the customer

        workspace_permission : typing.Optional[str]
            The workspace permission of the user. This is deprecated, use `seat_type` instead.

        seat_type : typing.Optional[SeatType]
            The seat type of the user

        group_ids : typing.Optional[typing.Sequence[str]]
            The group ids of the user

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddWorkspaceInviteResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.workspace.invites.create(
            email="john.doe@testmail.com",
        )
        """
        _response = self._raw_client.create(
            email=email,
            workspace_permission=workspace_permission,
            seat_type=seat_type,
            group_ids=group_ids,
            request_options=request_options,
        )
        return _response.data

    def create_batch(
        self,
        *,
        emails: typing.Sequence[str],
        seat_type: typing.Optional[SeatType] = OMIT,
        group_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddWorkspaceInviteResponseModel:
        """
        Sends email invitations to join your workspace to the provided emails. Requires all email addresses to be part of a verified domain. If the users don't have an account they will be prompted to create one. If the users accept these invites they will be added as users to your workspace and your subscription using one of your seats. This endpoint may only be called by workspace members with the WORKSPACE_MEMBERS_INVITE permission.

        Parameters
        ----------
        emails : typing.Sequence[str]
            The email of the customer

        seat_type : typing.Optional[SeatType]
            The seat type of the user

        group_ids : typing.Optional[typing.Sequence[str]]
            The group ids of the user

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddWorkspaceInviteResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.workspace.invites.create_batch(
            emails=["emails"],
        )
        """
        _response = self._raw_client.create_batch(
            emails=emails, seat_type=seat_type, group_ids=group_ids, request_options=request_options
        )
        return _response.data

    def delete(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteWorkspaceInviteResponseModel:
        """
        Invalidates an existing email invitation. The invitation will still show up in the inbox it has been delivered to, but activating it to join the workspace won't work. This endpoint may only be called by workspace members with the WORKSPACE_MEMBERS_INVITE permission.

        Parameters
        ----------
        email : str
            The email of the customer

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteWorkspaceInviteResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.workspace.invites.delete(
            email="john.doe@testmail.com",
        )
        """
        _response = self._raw_client.delete(email=email, request_options=request_options)
        return _response.data


class AsyncInvitesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawInvitesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawInvitesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawInvitesClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        email: str,
        workspace_permission: typing.Optional[str] = OMIT,
        seat_type: typing.Optional[SeatType] = OMIT,
        group_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddWorkspaceInviteResponseModel:
        """
        Sends an email invitation to join your workspace to the provided email. If the user doesn't have an account they will be prompted to create one. If the user accepts this invite they will be added as a user to your workspace and your subscription using one of your seats. This endpoint may only be called by workspace members with the WORKSPACE_MEMBERS_INVITE permission. If the user is already in the workspace a 400 error will be returned.

        Parameters
        ----------
        email : str
            The email of the customer

        workspace_permission : typing.Optional[str]
            The workspace permission of the user. This is deprecated, use `seat_type` instead.

        seat_type : typing.Optional[SeatType]
            The seat type of the user

        group_ids : typing.Optional[typing.Sequence[str]]
            The group ids of the user

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddWorkspaceInviteResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspace.invites.create(
                email="john.doe@testmail.com",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            email=email,
            workspace_permission=workspace_permission,
            seat_type=seat_type,
            group_ids=group_ids,
            request_options=request_options,
        )
        return _response.data

    async def create_batch(
        self,
        *,
        emails: typing.Sequence[str],
        seat_type: typing.Optional[SeatType] = OMIT,
        group_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddWorkspaceInviteResponseModel:
        """
        Sends email invitations to join your workspace to the provided emails. Requires all email addresses to be part of a verified domain. If the users don't have an account they will be prompted to create one. If the users accept these invites they will be added as users to your workspace and your subscription using one of your seats. This endpoint may only be called by workspace members with the WORKSPACE_MEMBERS_INVITE permission.

        Parameters
        ----------
        emails : typing.Sequence[str]
            The email of the customer

        seat_type : typing.Optional[SeatType]
            The seat type of the user

        group_ids : typing.Optional[typing.Sequence[str]]
            The group ids of the user

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddWorkspaceInviteResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspace.invites.create_batch(
                emails=["emails"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_batch(
            emails=emails, seat_type=seat_type, group_ids=group_ids, request_options=request_options
        )
        return _response.data

    async def delete(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteWorkspaceInviteResponseModel:
        """
        Invalidates an existing email invitation. The invitation will still show up in the inbox it has been delivered to, but activating it to join the workspace won't work. This endpoint may only be called by workspace members with the WORKSPACE_MEMBERS_INVITE permission.

        Parameters
        ----------
        email : str
            The email of the customer

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteWorkspaceInviteResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspace.invites.delete(
                email="john.doe@testmail.com",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(email=email, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class AgentResponsePayload(UncheckedBaseModel):
    """
    Text chunk sent from your server to ElevenLabs for speech synthesis.
    Stream LLM output by sending multiple messages with `is_final: false`, then
    terminate the response with a message where `is_final: true` and `content` is
    an empty string.
    """

    type: typing.Literal["agent_response"] = pydantic.Field(default="agent_response")
    """
    The message type identifier.
    """

    content: str = pydantic.Field()
    """
    The text to synthesize. For streaming responses, send incremental chunks here.
    The final message in a response must have an empty string (`""`).
    """

    event_id: typing.Optional[int] = pydantic.Field(default=None)
    """
    The `event_id` from the `user_transcript` this response addresses. ElevenLabs
    uses this to discard responses that belong to an interrupted turn.
    """

    is_final: bool = pydantic.Field()
    """
    Set to `true` on the last message of a response (with an empty `content`).
    Set to `false` on all preceding chunks.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import json
import logging
import re
import typing
from dataclasses import dataclass

from elevenlabs.core.request_options import RequestOptions
from elevenlabs.music.client import AsyncMusicClient as AutogeneratedAsyncMusicClient
from elevenlabs.music.client import MusicClient as AutogeneratedMusicClient
from elevenlabs.types.allowed_output_formats import AllowedOutputFormats
from elevenlabs.types.music_prompt import MusicPrompt

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)

logger = logging.getLogger(__name__)


def _find_audio_start(data: bytes, start: int) -> int:
    """Find the blank line separating headers from body, returning the index of the body start."""
    crlf_pos = data.find(b"\r\n\r\n", start)
    if crlf_pos != -1:
        return crlf_pos + 4

    lf_pos = data.find(b"\n\n", start)
    if lf_pos != -1:
        return lf_pos + 2

    raise ValueError("Could not find header/body separator in audio part")


@dataclass
class SongMetadata:
    title: str
    description: str
    genres: typing.List[str]
    languages: typing.List[str]
    is_explicit: bool


@dataclass
class MultipartResponse:
    json: typing.Dict[str, typing.Any]  # Contains compositionPlan and songMetadata
    audio: bytes
    filename: str
    song_id: typing.Optional[str] = None


class MusicClient(AutogeneratedMusicClient):
    """
    A client to handle ElevenLabs music-related functionality
    Extends the autogenerated client to include custom music methods
    """

    def compose_detailed(  # type: ignore[override]
        self,
        *,
        output_format: typing.Optional[AllowedOutputFormats] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[MusicPrompt] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[typing.Literal["music_v1"]] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        with_timestamps: typing.Optional[bool] = OMIT,
        sign_with_c2pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
        **kwargs: typing.Any,
    ) -> MultipartResponse:
        """
        Compose a song from a prompt or a composition plan with detailed response parsing.
        This method calls the original compose_detailed and then parses the stream response.

        Returns a MultipartResponse containing parsed JSON metadata, audio bytes, filename,
        and song_id (if store_for_inpainting was True).
        """
        # Use the raw client directly to access response headers (for song_id)
        with self._raw_client.compose_detailed(
            output_format=output_format,
            prompt=prompt,
            composition_plan=composition_plan,
            music_length_ms=music_length_ms,
            model_id=model_id,
            force_instrumental=force_instrumental,
            store_for_inpainting=store_for_inpainting,
            with_timestamps=with_timestamps,
            sign_with_c_2_pa=sign_with_c2pa,
            request_options=request_options,
        ) as r:
            result = self._parse_multipart(r.data)
            result.song_id = r.headers.get("song-id")
            return result

    def _parse_multipart(self, stream: typing.Iterator[bytes]) -> MultipartResponse:
        """
        Reads a byte stream containing multipart data and parses it into JSON and audio parts.

        Args:
            stream: Iterator of bytes from ElevenLabs music API response

        Returns:
            MultipartResponse containing parsed JSON metadata, audio bytes, and filename
        """
        # Collect all chunks into a single bytes object
        chunks = []
        for chunk in stream:
            chunks.append(chunk)

        # Combine all chunks into a single buffer
        response_bytes = b''.join(chunks)

        # Parse the multipart content
        response_text = response_bytes.decode('utf-8', errors='ignore')
        lines = response_text.split('\n')

        if not lines:
            raise ValueError("Empty response from music API")

        boundary = lines[0].strip()

        # Find the JSON part (should be early in the response)
        json_data = None
        filename = 'generated_music.mp3'

        # Parse JSON from the text representation
        for i in range(min(10, len(lines))):
            if 'Content-Type: application/json' in lines[i] and i + 2 < len(lines):
                json_line = lines[i + 2]
                if json_line.strip() and json_line.startswith('{'):
                    try:
                        json_data = json.loads(json_line)
                    except json.JSONDecodeError as e:
                        logger.error(f'Failed to parse composition plan JSON: {e}')
                    break

        # Extract filename from headers
        for i in range(min(20, len(lines))):
            if 'filename=' in lines[i]:
                match = re.search(r'filename="([^"]+)"', lines[i])
                if match:
                    filename = match.group(1)
                    break

        # Find where the audio data starts (after the second boundary and headers)
        boundary_bytes = boundary.encode('utf-8')
        first_boundary = -1
        second_boundary = -1

        for i in range(len(response_bytes) - len(boundary_bytes) + 1):
            if response_bytes[i:i + len(boundary_bytes)] == boundary_bytes:
                if first_boundary == -1:
                    first_boundary = i
                elif second_boundary == -1:
                    second_boundary = i
                    break

        if second_boundary == -1:
            raise ValueError('Could not find audio part boundary')

        # Find the start of audio data (after headers and empty line)
        audio_start = _find_audio_start(response_bytes, second_boundary + len(boundary_bytes))

        # Audio goes until the end (or until we find another boundary)
        audio_buffer = response_bytes[audio_start:]

        if not json_data:
            raise ValueError('Could not parse JSON data')

        return MultipartResponse(
            json=json_data,
            audio=audio_buffer,
            filename=filename
        )


class AsyncMusicClient(AutogeneratedAsyncMusicClient):
    """
    An async client to handle ElevenLabs music-related functionality
    Extends the autogenerated async client to include custom music methods
    """

    async def compose_detailed(  # type: ignore[override]
        self,
        *,
        output_format: typing.Optional[AllowedOutputFormats] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[MusicPrompt] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[typing.Literal["music_v1"]] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        with_timestamps: typing.Optional[bool] = OMIT,
        sign_with_c2pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
        **kwargs: typing.Any,
    ) -> MultipartResponse:
        """
        Compose a song from a prompt or a composition plan with detailed response parsing.
        This method calls the original compose_detailed and then parses the stream response.

        Returns a MultipartResponse containing parsed JSON metadata, audio bytes, filename,
        and song_id (if store_for_inpainting was True).
        """
        # Use the raw client directly to access response headers (for song_id)
        async with self._raw_client.compose_detailed(
            output_format=output_format,
            prompt=prompt,
            composition_plan=composition_plan,
            music_length_ms=music_length_ms,
            model_id=model_id,
            force_instrumental=force_instrumental,
            store_for_inpainting=store_for_inpainting,
            with_timestamps=with_timestamps,
            sign_with_c_2_pa=sign_with_c2pa,
            request_options=request_options,
        ) as r:
            result = await self._parse_multipart_async(r.data)
            result.song_id = r.headers.get("song-id")
            return result

    async def _parse_multipart_async(self, stream: typing.AsyncIterator[bytes]) -> MultipartResponse:
        """
        Reads an async byte stream containing multipart data and parses it into JSON and audio parts.

        Args:
            stream: AsyncIterator of bytes from ElevenLabs music API response

        Returns:
            MultipartResponse containing parsed JSON metadata, audio bytes, and filename
        """
        # Collect all chunks into a single bytes object
        chunks = []
        async for chunk in stream:
            chunks.append(chunk)

        # Combine all chunks into a single buffer
        response_bytes = b''.join(chunks)

        # Parse the multipart content
        response_text = response_bytes.decode('utf-8', errors='ignore')
        lines = response_text.split('\n')

        if not lines:
            raise ValueError("Empty response from music API")

        boundary = lines[0].strip()

        # Find the JSON part (should be early in the response)
        json_data = None
        filename = 'generated_music.mp3'

        # Parse JSON from the text representation
        for i in range(min(10, len(lines))):
            if 'Content-Type: application/json' in lines[i] and i + 2 < len(lines):
                json_line = lines[i + 2]
                if json_line.strip() and json_line.startswith('{'):
                    try:
                        json_data = json.loads(json_line)
                    except json.JSONDecodeError as e:
                        logger.error(f'Failed to parse JSON: {e}')
                    break

        # Extract filename from headers
        for i in range(min(20, len(lines))):
            if 'filename=' in lines[i]:
                match = re.search(r'filename="([^"]+)"', lines[i])
                if match:
                    filename = match.group(1)
                    break

        # Find where the audio data starts (after the second boundary and headers)
        boundary_bytes = boundary.encode('utf-8')
        first_boundary = -1
        second_boundary = -1

        for i in range(len(response_bytes) - len(boundary_bytes) + 1):
            if response_bytes[i:i + len(boundary_bytes)] == boundary_bytes:
                if first_boundary == -1:
                    first_boundary = i
                elif second_boundary == -1:
                    second_boundary = i
                    break

        if second_boundary == -1:
            raise ValueError('Could not find audio part boundary')

        # Find the start of audio data (after headers and empty line)
        audio_start = _find_audio_start(response_bytes, second_boundary + len(boundary_bytes))

        # Audio goes until the end (or until we find another boundary)
        audio_buffer = response_bytes[audio_start:]

        if not json_data:
            raise ValueError('Could not parse JSON data')

        return MultipartResponse(
            json=json_data,
            audio=audio_buffer,
            filename=filename
        )


# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .agent_test_entity_type import AgentTestEntityType
from .agent_test_folder_path_segment_response_model import AgentTestFolderPathSegmentResponseModel
from .conversation_initiation_source import ConversationInitiationSource
from .resource_access_info import ResourceAccessInfo
from .test_type import TestType


class UnitTestSummaryResponseModel(UncheckedBaseModel):
    id: str = pydantic.Field()
    """
    The ID of the test
    """

    name: str = pydantic.Field()
    """
    Name of the test
    """

    access_info: typing.Optional[ResourceAccessInfo] = pydantic.Field(default=None)
    """
    The access information of the test
    """

    created_at_unix_secs: int = pydantic.Field()
    """
    Creation time of the test in unix seconds
    """

    last_updated_at_unix_secs: int = pydantic.Field()
    """
    Last update time of the test in unix seconds
    """

    type: TestType = pydantic.Field()
    """
    Type of the test or entity
    """

    entity_type: typing.Optional[AgentTestEntityType] = pydantic.Field(default=None)
    """
    The type of entity (test or folder)
    """

    folder_parent_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the parent folder
    """

    folder_path: typing.Optional[typing.List[AgentTestFolderPathSegmentResponseModel]] = pydantic.Field(default=None)
    """
    The folder path segments from root to this entity
    """

    children_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of direct children (tests and subfolders) for folders only
    """

    conversation_initiation_source: typing.Optional[ConversationInitiationSource] = pydantic.Field(default=None)
    """
    Channel the test simulates the conversation as. Null for folders or default behavior.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .response_filter_mode import ResponseFilterMode


class ResponseFilter(UncheckedBaseModel):
    """
    Configuration for filtering tool responses before they are visible to the agent.
    """

    mode: typing.Optional[ResponseFilterMode] = pydantic.Field(default=None)
    """
    Controls how tool responses are filtered. 'all' returns entire response, 'allow' returns only specified paths, 'hide_all' hides the entire response.
    """

    filters: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Dot notation paths to include when mode is 'allow' (e.g., ['ticket.id', 'ticket.status']).
    """

    content_type: typing.Optional[typing.Literal["application/json"]] = pydantic.Field(default=None)
    """
    Content type for response filtering. Only 'application/json' responses are filtered.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.add_workspace_group_member_response_model import AddWorkspaceGroupMemberResponseModel
from ....types.delete_workspace_group_member_response_model import DeleteWorkspaceGroupMemberResponseModel
from .raw_client import AsyncRawMembersClient, RawMembersClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class MembersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMembersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawMembersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMembersClient
        """
        return self._raw_client

    def remove(
        self, group_id: str, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteWorkspaceGroupMemberResponseModel:
        """
        Removes a member from the specified group. Requires `group_members_manage` permission.

        Parameters
        ----------
        group_id : str
            The ID of the target group.

        email : str
            The email of the target workspace member.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteWorkspaceGroupMemberResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.workspace.groups.members.remove(
            group_id="group_id",
            email="email",
        )
        """
        _response = self._raw_client.remove(group_id, email=email, request_options=request_options)
        return _response.data

    def add(
        self, group_id: str, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AddWorkspaceGroupMemberResponseModel:
        """
        Adds a member of your workspace to the specified group. Requires `group_members_manage` permission.

        Parameters
        ----------
        group_id : str
            The ID of the target group.

        email : str
            The email of the target workspace member.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddWorkspaceGroupMemberResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.workspace.groups.members.add(
            group_id="group_id",
            email="email",
        )
        """
        _response = self._raw_client.add(group_id, email=email, request_options=request_options)
        return _response.data


class AsyncMembersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMembersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawMembersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMembersClient
        """
        return self._raw_client

    async def remove(
        self, group_id: str, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteWorkspaceGroupMemberResponseModel:
        """
        Removes a member from the specified group. Requires `group_members_manage` permission.

        Parameters
        ----------
        group_id : str
            The ID of the target group.

        email : str
            The email of the target workspace member.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteWorkspaceGroupMemberResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspace.groups.members.remove(
                group_id="group_id",
                email="email",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.remove(group_id, email=email, request_options=request_options)
        return _response.data

    async def add(
        self, group_id: str, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AddWorkspaceGroupMemberResponseModel:
        """
        Adds a member of your workspace to the specified group. Requires `group_members_manage` permission.

        Parameters
        ----------
        group_id : str
            The ID of the target group.

        email : str
            The email of the target workspace member.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddWorkspaceGroupMemberResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspace.groups.members.add(
                group_id="group_id",
                email="email",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add(group_id, email=email, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .seat_type import SeatType
from .subscription_response import SubscriptionResponse


class User(UncheckedBaseModel):
    user_id: str = pydantic.Field()
    """
    The unique identifier of the user.
    """

    subscription: SubscriptionResponse = pydantic.Field()
    """
    Details of the user's subscription.
    """

    is_new_user: bool = pydantic.Field()
    """
    Whether the user is new. This field is deprecated and will be removed in the future. Use 'created_at' instead.
    """

    xi_api_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    The API key of the user.
    """

    can_use_delayed_payment_methods: bool = pydantic.Field()
    """
    This field is deprecated and will be removed in a future major version. Instead use subscription.trust_on_invoice_creation.
    """

    is_onboarding_completed: bool = pydantic.Field()
    """
    Whether the user's onboarding is completed.
    """

    is_onboarding_checklist_completed: bool = pydantic.Field()
    """
    Whether the user's onboarding checklist is completed.
    """

    show_compliance_terms: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether to show compliance terms (ToS, Privacy Policy, biometric consent) during onboarding. Set for users signing up from the marketing site.
    """

    first_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    First name of the user.
    """

    is_api_key_hashed: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the user's API key is hashed.
    """

    xi_api_key_preview: typing.Optional[str] = pydantic.Field(default=None)
    """
    The preview of the user's API key.
    """

    referral_link_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The referral link code of the user.
    """

    partnerstack_partner_default_link: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Partnerstack partner default link of the user.
    """

    created_at: int = pydantic.Field()
    """
    The unix timestamp of the user's creation. 0 if the user was created before the unix timestamp was added.
    """

    seat_type: SeatType = pydantic.Field()
    """
    The seat type of the user.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



#!/usr/bin/env python3
"""
Script to verify all types in src/elevenlabs/types/ can be imported and instantiated.
Allows pydantic ValidationError but catches other errors (like circular imports).
Each type is tested in a separate Python process to avoid import cache issues.
"""
import subprocess
import sys
from pathlib import Path
from typing import Optional
from concurrent.futures import ThreadPoolExecutor, as_completed


def get_class_name_from_file(file_path: Path) -> Optional[str]:
    """Convert a file name to the expected class name using PascalCase."""
    # Remove .py extension
    name = file_path.stem

    # Skip special files
    if name.startswith('__'):
        return None

    # Convert snake_case to PascalCase
    parts = name.split('_')
    class_name = ''.join(word.capitalize() for word in parts)

    return class_name


def verify_type_import_in_subprocess(module_name: str, class_name: str) -> Optional[str]:
    """
    Test importing and instantiating a type in a separate subprocess.
    Returns error_message if failed, None if successful.
    """
    # Get project root (parent of tests directory)
    project_root = Path(__file__).parent.parent

    # Python code to test import in isolation
    test_code = f"""
import sys
from pydantic import ValidationError as PydanticValidationError

try:
    # Try to import the type
    from elevenlabs.types.{module_name} import {class_name}

    # Try to instantiate with no args (expect ValidationError)
    try:
        {class_name}()
    except PydanticValidationError:
        pass  # Expected - type requires arguments
    except TypeError:
        pass  # Some types might not be Pydantic models (enums, etc.)

    sys.exit(0)  # Success

except ImportError as e:
    print(f"ImportError: {{e}}", file=sys.stderr)
    sys.exit(1)
except Exception as e:
    print(f"{{type(e).__name__}}: {{e}}", file=sys.stderr)
    sys.exit(1)
"""

    try:
        result = subprocess.run(
            ["poetry", "run", "python", "-c", test_code],
            capture_output=True,
            text=True,
            timeout=30,
            cwd=str(project_root)
        )

        if result.returncode == 0:
            return None
        else:
            error_msg = result.stderr.strip() if result.stderr else "Unknown error"
            return error_msg

    except subprocess.TimeoutExpired:
        return "Timeout: Import took longer than 30 seconds"
    except Exception as e:
        return f"Subprocess error: {str(e)}"


def main():
    # Get project root (parent of tests directory)
    project_root = Path(__file__).parent.parent
    types_dir = project_root / "src/elevenlabs/types"

    if not types_dir.exists():
        print(f"Error: {types_dir} does not exist", file=sys.stderr)
        sys.exit(1)

    # Get all Python files
    py_files = sorted([f for f in types_dir.glob("*.py") if not f.name.startswith("__")])

    print(f"Testing {len(py_files)} type files in parallel...\n")

    failures = []
    successes = []

    # Test types in parallel for speed
    with ThreadPoolExecutor(max_workers=10) as executor:
        future_to_file = {}

        for py_file in py_files:
            module_name = py_file.stem
            class_name = get_class_name_from_file(py_file)

            if class_name is None:
                continue  # Skip special files

            future = executor.submit(verify_type_import_in_subprocess, module_name, class_name)
            future_to_file[future] = (module_name, class_name)

        for future in as_completed(future_to_file):
            module_name, class_name = future_to_file[future]
            try:
                error = future.result()

                if error is None:
                    successes.append((module_name, class_name))
                    print(f"✓ {module_name}.{class_name}")
                else:
                    failures.append((module_name, class_name, error))
                    print(f"✗ {module_name}.{class_name}")

            except Exception as e:
                failures.append((module_name, class_name, f"Test execution failed: {str(e)}"))
                print(f"✗ {module_name}.{class_name}")

    # Print summary
    print(f"\n{'='*80}")
    print(f"Summary: {len(successes)} passed, {len(failures)} failed")
    print(f"{'='*80}")

    if failures:
        print("\nFailed imports:")
        for module_name, class_name, error in failures:
            print(f"\n{module_name}.{class_name}:")
            # Indent error messages
            for line in error.split('\n'):
                print(f"  {line}")
        sys.exit(1)
    else:
        print("\nAll types can be imported successfully!")
        sys.exit(0)


if __name__ == "__main__":
    main()



# This file was auto-generated by Fern from our API Definition.

import typing

PendingSubscriptionSwitchResponseModelNextTier = typing.Union[
    typing.Literal[
        "free",
        "starter",
        "creator",
        "pro",
        "growing_business",
        "scale_2024_08_10",
        "grant_tier_1_2025_07_23",
        "grant_tier_2_2025_07_23",
        "trial",
        "enterprise",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .widget_text_contents import WidgetTextContents


class WidgetLanguagePresetResponse(UncheckedBaseModel):
    first_message: typing.Optional[str] = None
    text_contents: typing.Optional[WidgetTextContents] = pydantic.Field(default=None)
    """
    The text contents for the selected language
    """

    terms_text: typing.Optional[str] = pydantic.Field(default=None)
    """
    The text to display for terms and conditions in this language
    """

    terms_html: typing.Optional[str] = pydantic.Field(default=None)
    """
    The HTML to display for terms and conditions in this language
    """

    terms_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    The key to display for terms and conditions in this language
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import asyncio

from elevenlabs import VoiceSettings, Voice
from elevenlabs.play import play
from elevenlabs.client import AsyncElevenLabs, ElevenLabs

from .utils import IN_GITHUB, DEFAULT_TEXT, DEFAULT_VOICE, DEFAULT_MODEL
import base64


def test_tts_convert() -> None:
    """Test basic text-to-speech generation."""
    client = ElevenLabs()
    audio_generator = client.text_to_speech.convert(text=DEFAULT_TEXT, voice_id=DEFAULT_VOICE, model_id=DEFAULT_MODEL)
    audio = b"".join(audio_generator)
    assert isinstance(audio, bytes), "TTS should return bytes"
    if not IN_GITHUB:
        play(audio)


def test_tts_convert_with_voice_settings() -> None:
    """Test TTS with custom voice settings."""
    client = ElevenLabs()
    audio_generator = client.text_to_speech.convert(
        text=DEFAULT_TEXT,
        voice_id=DEFAULT_VOICE,
        model_id=DEFAULT_MODEL,
        voice_settings=VoiceSettings(stability=0.71, similarity_boost=0.5, style=0.0, use_speaker_boost=True),
    )
    audio = b"".join(audio_generator)
    assert isinstance(audio, bytes), "TTS with voice settings should return bytes"
    if not IN_GITHUB:
        play(audio)


def test_tts_convert_as_stream():
    async def main():
        async_client = AsyncElevenLabs()
        results = async_client.text_to_speech.stream(
            text=DEFAULT_TEXT, voice_id=DEFAULT_VOICE, model_id=DEFAULT_MODEL
        )
        out = b""
        async for value in results:
            assert isinstance(value, bytes), "Stream chunks should be bytes"
            out += value
        if not IN_GITHUB:
            play(out)

    asyncio.run(main())


def test_tts_convert_with_timestamps() -> None:
    """Test TTS generation with timestamps."""
    client = ElevenLabs()
    result = client.text_to_speech.convert_with_timestamps(
        text=DEFAULT_TEXT, voice_id=DEFAULT_VOICE, model_id=DEFAULT_MODEL
    )

    # Check that the alignment data exists and has the expected structure
    assert hasattr(result, 'alignment')
    assert hasattr(result, 'audio_base_64')

    # Verify alignment contains timing data
    assert result.alignment is not None
    assert len(result.alignment.character_start_times_seconds) > 0
    assert len(result.alignment.character_end_times_seconds) > 0

    if not IN_GITHUB:
        audio_bytes = base64.b64decode(result.audio_base_64)
        play(audio_bytes)


def test_tts_stream_with_timestamps():
    async def main():
        async_client = AsyncElevenLabs()
        audio_data = b""
        async_stream = async_client.text_to_speech.stream_with_timestamps(
            voice_id=DEFAULT_VOICE,
            text=DEFAULT_TEXT,
            model_id=DEFAULT_MODEL,
        )
        async for chunk in async_stream:
            if hasattr(chunk, "audio_base_64"):
                audio_bytes = base64.b64decode(chunk.audio_base_64)
                audio_data += audio_bytes

        if not IN_GITHUB:
            play(audio_data)

    asyncio.run(main())



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.unchecked_base_model import UncheckedBaseModel


class AstGreaterThanOperatorNodeInput(UncheckedBaseModel):
    left: "AstNodeInput" = pydantic.Field()
    """
    Left operand of the binary operator.
    """

    right: "AstNodeInput" = pydantic.Field()
    """
    Right operand of the binary operator.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .ast_node_input import AstNodeInput  # noqa: E402, I001

update_forward_refs(AstGreaterThanOperatorNodeInput)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .open_aimcp_tool_require_approval import OpenAimcpToolRequireApproval


class OpenAimcpTool(UncheckedBaseModel):
    server_label: str
    server_url: str
    authorization: typing.Optional[str] = None
    allowed_tools: typing.Optional[typing.List[str]] = None
    require_approval: typing.Optional[OpenAimcpToolRequireApproval] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

CaptionStyleWordAnimationModelEnterType = typing.Union[
    typing.Literal["none", "fade", "scale", "pop", "slide_up", "slide_down"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.get_tool_dependent_agents_response_model import GetToolDependentAgentsResponseModel
from ...types.sort_direction import SortDirection
from ...types.tool_request_model import ToolRequestModel
from ...types.tool_response_model import ToolResponseModel
from ...types.tool_sort_by import ToolSortBy
from ...types.tool_type_filter import ToolTypeFilter
from ...types.tools_response_model import ToolsResponseModel

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawToolsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        search: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        show_only_owned_documents: typing.Optional[bool] = None,
        created_by_user_id: typing.Optional[str] = None,
        types: typing.Optional[typing.Union[ToolTypeFilter, typing.Sequence[ToolTypeFilter]]] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        sort_by: typing.Optional[ToolSortBy] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ToolsResponseModel]:
        """
        Get all available tools in the workspace.

        Parameters
        ----------
        search : typing.Optional[str]
            If specified, the endpoint returns only tools whose names start with this string.

        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100, defaults to 30.

        show_only_owned_documents : typing.Optional[bool]
            If set to true, the endpoint will return only tools owned by you (and not shared from somebody else). Deprecated: use created_by_user_id instead.

        created_by_user_id : typing.Optional[str]
            Filter tools by creator user ID. When set, only tools created by this user are returned. Takes precedence over show_only_owned_documents. Use '@me' to refer to the authenticated user.

        types : typing.Optional[typing.Union[ToolTypeFilter, typing.Sequence[ToolTypeFilter]]]
            If present, the endpoint will return only tools of the given types.

        sort_direction : typing.Optional[SortDirection]
            The direction to sort the results

        sort_by : typing.Optional[ToolSortBy]
            The field to sort the results by

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ToolsResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/convai/tools",
            method="GET",
            params={
                "search": search,
                "page_size": page_size,
                "show_only_owned_documents": show_only_owned_documents,
                "created_by_user_id": created_by_user_id,
                "types": types,
                "sort_direction": sort_direction,
                "sort_by": sort_by,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolsResponseModel,
                    construct_type(
                        type_=ToolsResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self, *, request: ToolRequestModel, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ToolResponseModel]:
        """
        Add a new tool to the available tools in the workspace.

        Parameters
        ----------
        request : ToolRequestModel

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ToolResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/convai/tools",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=ToolRequestModel, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolResponseModel,
                    construct_type(
                        type_=ToolResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, tool_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ToolResponseModel]:
        """
        Get tool that is available in the workspace.

        Parameters
        ----------
        tool_id : str
            ID of the requested tool.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ToolResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/tools/{jsonable_encoder(tool_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolResponseModel,
                    construct_type(
                        type_=ToolResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self,
        tool_id: str,
        *,
        force: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.Any]:
        """
        Delete tool from the workspace.

        Parameters
        ----------
        tool_id : str
            ID of the requested tool.

        force : typing.Optional[bool]
            If set to true, the tool will be deleted regardless of whether it is used by any agents and it will be removed from the dependent agents and branches.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Any]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/tools/{jsonable_encoder(tool_id)}",
            method="DELETE",
            params={
                "force": force,
            },
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    construct_type(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self, tool_id: str, *, request: ToolRequestModel, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ToolResponseModel]:
        """
        Update tool that is available in the workspace.

        Parameters
        ----------
        tool_id : str
            ID of the requested tool.

        request : ToolRequestModel

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ToolResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/tools/{jsonable_encoder(tool_id)}",
            method="PATCH",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=ToolRequestModel, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolResponseModel,
                    construct_type(
                        type_=ToolResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_dependent_agents(
        self,
        tool_id: str,
        *,
        cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetToolDependentAgentsResponseModel]:
        """
        Get a list of agents depending on this tool

        Parameters
        ----------
        tool_id : str
            ID of the requested tool.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100, defaults to 30.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetToolDependentAgentsResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/tools/{jsonable_encoder(tool_id)}/dependent-agents",
            method="GET",
            params={
                "cursor": cursor,
                "page_size": page_size,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetToolDependentAgentsResponseModel,
                    construct_type(
                        type_=GetToolDependentAgentsResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawToolsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        search: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        show_only_owned_documents: typing.Optional[bool] = None,
        created_by_user_id: typing.Optional[str] = None,
        types: typing.Optional[typing.Union[ToolTypeFilter, typing.Sequence[ToolTypeFilter]]] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        sort_by: typing.Optional[ToolSortBy] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ToolsResponseModel]:
        """
        Get all available tools in the workspace.

        Parameters
        ----------
        search : typing.Optional[str]
            If specified, the endpoint returns only tools whose names start with this string.

        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100, defaults to 30.

        show_only_owned_documents : typing.Optional[bool]
            If set to true, the endpoint will return only tools owned by you (and not shared from somebody else). Deprecated: use created_by_user_id instead.

        created_by_user_id : typing.Optional[str]
            Filter tools by creator user ID. When set, only tools created by this user are returned. Takes precedence over show_only_owned_documents. Use '@me' to refer to the authenticated user.

        types : typing.Optional[typing.Union[ToolTypeFilter, typing.Sequence[ToolTypeFilter]]]
            If present, the endpoint will return only tools of the given types.

        sort_direction : typing.Optional[SortDirection]
            The direction to sort the results

        sort_by : typing.Optional[ToolSortBy]
            The field to sort the results by

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ToolsResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/convai/tools",
            method="GET",
            params={
                "search": search,
                "page_size": page_size,
                "show_only_owned_documents": show_only_owned_documents,
                "created_by_user_id": created_by_user_id,
                "types": types,
                "sort_direction": sort_direction,
                "sort_by": sort_by,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolsResponseModel,
                    construct_type(
                        type_=ToolsResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self, *, request: ToolRequestModel, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ToolResponseModel]:
        """
        Add a new tool to the available tools in the workspace.

        Parameters
        ----------
        request : ToolRequestModel

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ToolResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/convai/tools",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=ToolRequestModel, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolResponseModel,
                    construct_type(
                        type_=ToolResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, tool_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ToolResponseModel]:
        """
        Get tool that is available in the workspace.

        Parameters
        ----------
        tool_id : str
            ID of the requested tool.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ToolResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/tools/{jsonable_encoder(tool_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolResponseModel,
                    construct_type(
                        type_=ToolResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self,
        tool_id: str,
        *,
        force: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.Any]:
        """
        Delete tool from the workspace.

        Parameters
        ----------
        tool_id : str
            ID of the requested tool.

        force : typing.Optional[bool]
            If set to true, the tool will be deleted regardless of whether it is used by any agents and it will be removed from the dependent agents and branches.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Any]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/tools/{jsonable_encoder(tool_id)}",
            method="DELETE",
            params={
                "force": force,
            },
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    construct_type(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self, tool_id: str, *, request: ToolRequestModel, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ToolResponseModel]:
        """
        Update tool that is available in the workspace.

        Parameters
        ----------
        tool_id : str
            ID of the requested tool.

        request : ToolRequestModel

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ToolResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/tools/{jsonable_encoder(tool_id)}",
            method="PATCH",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=ToolRequestModel, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ToolResponseModel,
                    construct_type(
                        type_=ToolResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_dependent_agents(
        self,
        tool_id: str,
        *,
        cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetToolDependentAgentsResponseModel]:
        """
        Get a list of agents depending on this tool

        Parameters
        ----------
        tool_id : str
            ID of the requested tool.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100, defaults to 30.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetToolDependentAgentsResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/tools/{jsonable_encoder(tool_id)}/dependent-agents",
            method="GET",
            params={
                "cursor": cursor,
                "page_size": page_size,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetToolDependentAgentsResponseModel,
                    construct_type(
                        type_=GetToolDependentAgentsResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



import typing

from .core import RequestOptions
from .speech_engine.client import AsyncSpeechEngineClient as AutogeneratedAsyncSpeechEngineClient
from .speech_engine.client import SpeechEngineClient as AutogeneratedSpeechEngineClient
from .speech_engine.resource import SpeechEngineResource
from .types.agent_call_limits import AgentCallLimits
from .types.asr_conversational_config import AsrConversationalConfig
from .types.base_turn_config import BaseTurnConfig
from .types.conversation_config_input import ConversationConfigInput
from .types.privacy_config_input import PrivacyConfigInput
from .types.speech_engine_config import SpeechEngineConfig
from .types.speech_engine_conversation_initiation_client_data_config import SpeechEngineConversationInitiationClientDataConfig
from .types.tts_conversational_config_input import TtsConversationalConfigInput

OMIT = typing.cast(typing.Any, ...)


class SpeechEngineClient(AutogeneratedSpeechEngineClient):
    """Extends the generated SpeechEngineClient with WebSocket server integration."""

    def create(  # type: ignore[override]
        self,
        *,
        speech_engine: SpeechEngineConfig,
        name: typing.Optional[str] = OMIT,
        asr: typing.Optional[AsrConversationalConfig] = OMIT,
        tts: typing.Optional[TtsConversationalConfigInput] = OMIT,
        turn: typing.Optional[BaseTurnConfig] = OMIT,
        conversation: typing.Optional[ConversationConfigInput] = OMIT,
        privacy: typing.Optional[PrivacyConfigInput] = OMIT,
        call_limits: typing.Optional[AgentCallLimits] = OMIT,
        language: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        overrides: typing.Optional[SpeechEngineConversationInitiationClientDataConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SpeechEngineResource:
        """Create a Speech Engine resource.

        Makes an API call to create the engine, then returns a
        :class:`SpeechEngineResource` with WebSocket integration methods
        (:meth:`~SpeechEngineResource.serve`,
        :meth:`~SpeechEngineResource.create_session`,
        :meth:`~SpeechEngineResource.verify_request`).
        """
        response = super().create(
            speech_engine=speech_engine,
            name=name,
            asr=asr,
            tts=tts,
            turn=turn,
            conversation=conversation,
            privacy=privacy,
            call_limits=call_limits,
            language=language,
            tags=tags,
            overrides=overrides,
            request_options=request_options,
        )
        return SpeechEngineResource(
            engine_id=response.speech_engine_id,
            client_wrapper=self._raw_client._client_wrapper,
            response=response,
        )

    def get(  # type: ignore[override]
        self,
        speech_engine_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SpeechEngineResource:
        """Retrieve a Speech Engine resource.

        Makes an API call to validate the engine exists, then returns a
        :class:`SpeechEngineResource` with WebSocket integration methods
        (:meth:`~SpeechEngineResource.serve`,
        :meth:`~SpeechEngineResource.create_session`,
        :meth:`~SpeechEngineResource.verify_request`).
        """
        response = super().get(speech_engine_id, request_options=request_options)
        return SpeechEngineResource(
            engine_id=speech_engine_id,
            client_wrapper=self._raw_client._client_wrapper,
            response=response,
        )

    def update(  # type: ignore[override]
        self,
        speech_engine_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        speech_engine: typing.Optional[SpeechEngineConfig] = OMIT,
        asr: typing.Optional[AsrConversationalConfig] = OMIT,
        tts: typing.Optional[TtsConversationalConfigInput] = OMIT,
        turn: typing.Optional[BaseTurnConfig] = OMIT,
        conversation: typing.Optional[ConversationConfigInput] = OMIT,
        privacy: typing.Optional[PrivacyConfigInput] = OMIT,
        call_limits: typing.Optional[AgentCallLimits] = OMIT,
        language: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SpeechEngineResource:
        """Update a Speech Engine resource.

        Makes an API call to update the engine, then returns a
        :class:`SpeechEngineResource` with WebSocket integration methods
        (:meth:`~SpeechEngineResource.serve`,
        :meth:`~SpeechEngineResource.create_session`,
        :meth:`~SpeechEngineResource.verify_request`).
        """
        response = super().update(
            speech_engine_id,
            name=name,
            speech_engine=speech_engine,
            asr=asr,
            tts=tts,
            turn=turn,
            conversation=conversation,
            privacy=privacy,
            call_limits=call_limits,
            language=language,
            tags=tags,
            request_options=request_options,
        )
        return SpeechEngineResource(
            engine_id=speech_engine_id,
            client_wrapper=self._raw_client._client_wrapper,
            response=response,
        )


class AsyncSpeechEngineClient(AutogeneratedAsyncSpeechEngineClient):
    """Extends the generated AsyncSpeechEngineClient with WebSocket server integration."""

    async def create(  # type: ignore[override]
        self,
        *,
        speech_engine: SpeechEngineConfig,
        name: typing.Optional[str] = OMIT,
        asr: typing.Optional[AsrConversationalConfig] = OMIT,
        tts: typing.Optional[TtsConversationalConfigInput] = OMIT,
        turn: typing.Optional[BaseTurnConfig] = OMIT,
        conversation: typing.Optional[ConversationConfigInput] = OMIT,
        privacy: typing.Optional[PrivacyConfigInput] = OMIT,
        call_limits: typing.Optional[AgentCallLimits] = OMIT,
        language: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        overrides: typing.Optional[SpeechEngineConversationInitiationClientDataConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SpeechEngineResource:
        """Create a Speech Engine resource.

        Makes an API call to create the engine, then returns a
        :class:`SpeechEngineResource` with WebSocket integration methods
        (:meth:`~SpeechEngineResource.serve`,
        :meth:`~SpeechEngineResource.create_session`,
        :meth:`~SpeechEngineResource.verify_request`).
        """
        response = await super().create(
            speech_engine=speech_engine,
            name=name,
            asr=asr,
            tts=tts,
            turn=turn,
            conversation=conversation,
            privacy=privacy,
            call_limits=call_limits,
            language=language,
            tags=tags,
            overrides=overrides,
            request_options=request_options,
        )
        return SpeechEngineResource(
            engine_id=response.speech_engine_id,
            client_wrapper=self._raw_client._client_wrapper,
            response=response,
        )

    async def get(  # type: ignore[override]
        self,
        speech_engine_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SpeechEngineResource:
        """Retrieve a Speech Engine resource.

        Makes an API call to validate the engine exists, then returns a
        :class:`SpeechEngineResource` with WebSocket integration methods
        (:meth:`~SpeechEngineResource.serve`,
        :meth:`~SpeechEngineResource.create_session`,
        :meth:`~SpeechEngineResource.verify_request`).
        """
        response = await super().get(speech_engine_id, request_options=request_options)
        return SpeechEngineResource(
            engine_id=speech_engine_id,
            client_wrapper=self._raw_client._client_wrapper,
            response=response,
        )

    async def update(  # type: ignore[override]
        self,
        speech_engine_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        speech_engine: typing.Optional[SpeechEngineConfig] = OMIT,
        asr: typing.Optional[AsrConversationalConfig] = OMIT,
        tts: typing.Optional[TtsConversationalConfigInput] = OMIT,
        turn: typing.Optional[BaseTurnConfig] = OMIT,
        conversation: typing.Optional[ConversationConfigInput] = OMIT,
        privacy: typing.Optional[PrivacyConfigInput] = OMIT,
        call_limits: typing.Optional[AgentCallLimits] = OMIT,
        language: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SpeechEngineResource:
        """Update a Speech Engine resource.

        Makes an API call to update the engine, then returns a
        :class:`SpeechEngineResource` with WebSocket integration methods
        (:meth:`~SpeechEngineResource.serve`,
        :meth:`~SpeechEngineResource.create_session`,
        :meth:`~SpeechEngineResource.verify_request`).
        """
        response = await super().update(
            speech_engine_id,
            name=name,
            speech_engine=speech_engine,
            asr=asr,
            tts=tts,
            turn=turn,
            conversation=conversation,
            privacy=privacy,
            call_limits=call_limits,
            language=language,
            tags=tags,
            request_options=request_options,
        )
        return SpeechEngineResource(
            engine_id=speech_engine_id,
            client_wrapper=self._raw_client._client_wrapper,
            response=response,
        )



# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.committed_transcript_payload import CommittedTranscriptPayload
from ...types.committed_transcript_with_timestamps_payload import CommittedTranscriptWithTimestampsPayload
from ...types.partial_transcript_payload import PartialTranscriptPayload
from ...types.scribe_auth_error_payload import ScribeAuthErrorPayload
from ...types.scribe_chunk_size_exceeded_error_payload import ScribeChunkSizeExceededErrorPayload
from ...types.scribe_error_payload import ScribeErrorPayload
from ...types.scribe_input_error_payload import ScribeInputErrorPayload
from ...types.scribe_insufficient_audio_activity_error_payload import ScribeInsufficientAudioActivityErrorPayload
from ...types.scribe_queue_overflow_error_payload import ScribeQueueOverflowErrorPayload
from ...types.scribe_quota_exceeded_error_payload import ScribeQuotaExceededErrorPayload
from ...types.scribe_rate_limited_error_payload import ScribeRateLimitedErrorPayload
from ...types.scribe_resource_exhausted_error_payload import ScribeResourceExhaustedErrorPayload
from ...types.scribe_session_time_limit_exceeded_error_payload import ScribeSessionTimeLimitExceededErrorPayload
from ...types.scribe_throttled_error_payload import ScribeThrottledErrorPayload
from ...types.scribe_transcriber_error_payload import ScribeTranscriberErrorPayload
from ...types.scribe_unaccepted_terms_error_payload import ScribeUnacceptedTermsErrorPayload
from ...types.session_started_payload import SessionStartedPayload

ReceiveTranscription = typing.Union[
    SessionStartedPayload,
    PartialTranscriptPayload,
    CommittedTranscriptPayload,
    CommittedTranscriptWithTimestampsPayload,
    ScribeErrorPayload,
    ScribeAuthErrorPayload,
    ScribeQuotaExceededErrorPayload,
    ScribeThrottledErrorPayload,
    ScribeUnacceptedTermsErrorPayload,
    ScribeRateLimitedErrorPayload,
    ScribeQueueOverflowErrorPayload,
    ScribeResourceExhaustedErrorPayload,
    ScribeSessionTimeLimitExceededErrorPayload,
    ScribeInputErrorPayload,
    ScribeChunkSizeExceededErrorPayload,
    ScribeInsufficientAudioActivityErrorPayload,
    ScribeTranscriberErrorPayload,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .... import core
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.media_id import MediaId
from ....types.order_id import OrderId
from ....types.order_media_response import OrderMediaResponse
from ....types.register_media_response import RegisterMediaResponse
from .raw_client import AsyncRawMediaClient, RawMediaClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class MediaClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMediaClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawMediaClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMediaClient
        """
        return self._raw_client

    def register(
        self,
        order_id: OrderId,
        *,
        declared_language: str,
        media: typing.Optional[core.File] = OMIT,
        media_url: typing.Optional[str] = OMIT,
        media_url_filename: typing.Optional[str] = OMIT,
        media_url_content_type: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RegisterMediaResponse:
        """
        Registers a media file with an order, either by uploading it directly or by providing a URL to fetch it from. Exactly one of `media` or `media_url` must be provided. The registered media can then be referenced when adding order items.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order to which this media will be attached.

        declared_language : str
            The language code of the media content (e.g. 'en', 'es-ES'). Must be a supported source language for some order item kind.

        media : typing.Optional[core.File]
            See core.File for more documentation

        media_url : typing.Optional[str]
            A URL to fetch the media file from. Mutually exclusive with media.

        media_url_filename : typing.Optional[str]
            The filename for URL-sourced media (e.g. 'example.mp4'). Required when using media_url.

        media_url_content_type : typing.Optional[str]
            The MIME type for URL-sourced media (e.g. 'video/mp4'). Required when using media_url.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RegisterMediaResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.productions.orders.media.register(
            order_id="order_id",
            declared_language="declared_language",
        )
        """
        _response = self._raw_client.register(
            order_id,
            declared_language=declared_language,
            media=media,
            media_url=media_url,
            media_url_filename=media_url_filename,
            media_url_content_type=media_url_content_type,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, order_id: OrderId, media_id: MediaId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> OrderMediaResponse:
        """
        Retrieves metadata and a time-limited download URL for a previously uploaded media file.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        media_id : MediaId
            The ID of the media file.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrderMediaResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.productions.orders.media.get(
            order_id="order_id",
            media_id="media_id",
        )
        """
        _response = self._raw_client.get(order_id, media_id, request_options=request_options)
        return _response.data


class AsyncMediaClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMediaClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawMediaClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMediaClient
        """
        return self._raw_client

    async def register(
        self,
        order_id: OrderId,
        *,
        declared_language: str,
        media: typing.Optional[core.File] = OMIT,
        media_url: typing.Optional[str] = OMIT,
        media_url_filename: typing.Optional[str] = OMIT,
        media_url_content_type: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RegisterMediaResponse:
        """
        Registers a media file with an order, either by uploading it directly or by providing a URL to fetch it from. Exactly one of `media` or `media_url` must be provided. The registered media can then be referenced when adding order items.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order to which this media will be attached.

        declared_language : str
            The language code of the media content (e.g. 'en', 'es-ES'). Must be a supported source language for some order item kind.

        media : typing.Optional[core.File]
            See core.File for more documentation

        media_url : typing.Optional[str]
            A URL to fetch the media file from. Mutually exclusive with media.

        media_url_filename : typing.Optional[str]
            The filename for URL-sourced media (e.g. 'example.mp4'). Required when using media_url.

        media_url_content_type : typing.Optional[str]
            The MIME type for URL-sourced media (e.g. 'video/mp4'). Required when using media_url.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RegisterMediaResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.productions.orders.media.register(
                order_id="order_id",
                declared_language="declared_language",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.register(
            order_id,
            declared_language=declared_language,
            media=media,
            media_url=media_url,
            media_url_filename=media_url_filename,
            media_url_content_type=media_url_content_type,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, order_id: OrderId, media_id: MediaId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> OrderMediaResponse:
        """
        Retrieves metadata and a time-limited download URL for a previously uploaded media file.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        media_id : MediaId
            The ID of the media file.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrderMediaResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.productions.orders.media.get(
                order_id="order_id",
                media_id="media_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(order_id, media_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.get_conversation_users_page_response_model import GetConversationUsersPageResponseModel
from ...types.users_sort_by import UsersSortBy
from .raw_client import AsyncRawUsersClient, RawUsersClient


class UsersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUsersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawUsersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUsersClient
        """
        return self._raw_client

    def list(
        self,
        *,
        agent_id: typing.Optional[str] = None,
        branch_id: typing.Optional[str] = None,
        call_start_before_unix: typing.Optional[int] = None,
        call_start_after_unix: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        sort_by: typing.Optional[UsersSortBy] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetConversationUsersPageResponseModel:
        """
        Get distinct users from conversations with pagination.

        Parameters
        ----------
        agent_id : typing.Optional[str]
            Agent id (agent_…) or speech engine external id (seng_), resolved to the same underlying resource.

        branch_id : typing.Optional[str]
            Filter conversations by branch ID.

        call_start_before_unix : typing.Optional[int]
            Unix timestamp (in seconds) to filter conversations up to this start date.

        call_start_after_unix : typing.Optional[int]
            Unix timestamp (in seconds) to filter conversations after to this start date.

        search : typing.Optional[str]
            Search/filter by user ID (exact match).

        page_size : typing.Optional[int]
            How many users to return at maximum. Defaults to 30.

        sort_by : typing.Optional[UsersSortBy]
            The field to sort the results by. Defaults to last_contact_unix_secs.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetConversationUsersPageResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.users.list(
            agent_id="agent_id",
            branch_id="branch_id",
            call_start_before_unix=1,
            call_start_after_unix=1,
            search="search",
            page_size=1,
            sort_by="last_contact_unix_secs",
            cursor="cursor",
        )
        """
        _response = self._raw_client.list(
            agent_id=agent_id,
            branch_id=branch_id,
            call_start_before_unix=call_start_before_unix,
            call_start_after_unix=call_start_after_unix,
            search=search,
            page_size=page_size,
            sort_by=sort_by,
            cursor=cursor,
            request_options=request_options,
        )
        return _response.data


class AsyncUsersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUsersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawUsersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUsersClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        agent_id: typing.Optional[str] = None,
        branch_id: typing.Optional[str] = None,
        call_start_before_unix: typing.Optional[int] = None,
        call_start_after_unix: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        sort_by: typing.Optional[UsersSortBy] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetConversationUsersPageResponseModel:
        """
        Get distinct users from conversations with pagination.

        Parameters
        ----------
        agent_id : typing.Optional[str]
            Agent id (agent_…) or speech engine external id (seng_), resolved to the same underlying resource.

        branch_id : typing.Optional[str]
            Filter conversations by branch ID.

        call_start_before_unix : typing.Optional[int]
            Unix timestamp (in seconds) to filter conversations up to this start date.

        call_start_after_unix : typing.Optional[int]
            Unix timestamp (in seconds) to filter conversations after to this start date.

        search : typing.Optional[str]
            Search/filter by user ID (exact match).

        page_size : typing.Optional[int]
            How many users to return at maximum. Defaults to 30.

        sort_by : typing.Optional[UsersSortBy]
            The field to sort the results by. Defaults to last_contact_unix_secs.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetConversationUsersPageResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.users.list(
                agent_id="agent_id",
                branch_id="branch_id",
                call_start_before_unix=1,
                call_start_after_unix=1,
                search="search",
                page_size=1,
                sort_by="last_contact_unix_secs",
                cursor="cursor",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            agent_id=agent_id,
            branch_id=branch_id,
            call_start_before_unix=call_start_before_unix,
            call_start_after_unix=call_start_after_unix,
            search=search,
            page_size=page_size,
            sort_by=sort_by,
            cursor=cursor,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .dynamic_variable_value_type_output import DynamicVariableValueTypeOutput


class DynamicVariablesConfigWorkflowOverrideOutput(UncheckedBaseModel):
    dynamic_variable_placeholders: typing.Optional[
        typing.Dict[str, typing.Optional[DynamicVariableValueTypeOutput]]
    ] = pydantic.Field(default=None)
    """
    A dictionary of dynamic variable placeholders and their values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .unit_test_tool_call_parameter import UnitTestToolCallParameter


class ToolResponseMockConfigOutput(UncheckedBaseModel):
    parameter_conditions: typing.Optional[typing.List[UnitTestToolCallParameter]] = pydantic.Field(default=None)
    """
    If the list is empty, the mock will always activate.
    """

    mock_result: str = pydantic.Field()
    """
    The return value the LLM sees when this mock is active.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .literal_json_schema_property import LiteralJsonSchemaProperty

if typing.TYPE_CHECKING:
    from .array_json_schema_property_input import ArrayJsonSchemaPropertyInput
    from .object_json_schema_property_input import ObjectJsonSchemaPropertyInput
ObjectJsonSchemaPropertyInputPropertiesValue = typing.Union[
    LiteralJsonSchemaProperty, "ObjectJsonSchemaPropertyInput", "ArrayJsonSchemaPropertyInput"
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.unchecked_base_model import UncheckedBaseModel
from .bucketing_status import BucketingStatus
from .test_run_result_summary import TestRunResultSummary
from .unit_test_run_response_model import UnitTestRunResponseModel


class GetTestSuiteInvocationResponseModel(UncheckedBaseModel):
    id: str
    agent_id: typing.Optional[str] = None
    branch_id: typing.Optional[str] = None
    created_at: typing.Optional[int] = None
    folder_id: typing.Optional[str] = None
    repeat_count: typing.Optional[int] = None
    bucketing_status: typing.Optional[BucketingStatus] = pydantic.Field(default=None)
    """
    None when repeat_count==1 (no bucketing). Otherwise tracks bucketing lifecycle.
    """

    result_groups: typing.Optional[typing.List[TestRunResultSummary]] = None
    test_runs: typing.List[UnitTestRunResponseModel]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(GetTestSuiteInvocationResponseModel)



# This file was auto-generated by Fern from our API Definition.

import typing

ReviewResponseModelRejectReasonsItem = typing.Union[
    typing.Literal[
        "lacks_structure",
        "doesnt_open",
        "not_literary_work",
        "language_not_supported",
        "too_short",
        "duplicate",
        "promotional",
        "formatting_issues",
        "low_quality",
        "metadata_incomplete",
        "metadata_inaccurate",
        "typos",
        "review_error",
        "spam",
        "legal_violation",
        "content_policy",
        "public_domain",
        "other",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.create_pronunciation_dictionary_response_model import CreatePronunciationDictionaryResponseModel
from ....types.pronunciation_dictionary_version_locator import PronunciationDictionaryVersionLocator
from .raw_client import AsyncRawPronunciationDictionariesClient, RawPronunciationDictionariesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PronunciationDictionariesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPronunciationDictionariesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPronunciationDictionariesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPronunciationDictionariesClient
        """
        return self._raw_client

    def create(
        self,
        project_id: str,
        *,
        pronunciation_dictionary_locators: typing.Sequence[PronunciationDictionaryVersionLocator],
        invalidate_affected_text: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreatePronunciationDictionaryResponseModel:
        """
        Create a set of pronunciation dictionaries acting on a project. This will automatically mark text within this project as requiring reconverting where the new dictionary would apply or the old one no longer does.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        pronunciation_dictionary_locators : typing.Sequence[PronunciationDictionaryVersionLocator]
            A list of pronunciation dictionary locators (pronunciation_dictionary_id, version_id) encoded as a list of JSON strings for pronunciation dictionaries to be applied to the text. A list of json encoded strings is required as adding projects may occur through formData as opposed to jsonBody. To specify multiple dictionaries use multiple --form lines in your curl, such as --form 'pronunciation_dictionary_locators="{\\"pronunciation_dictionary_id\\":\\"Vmd4Zor6fplcA7WrINey\\",\\"version_id\\":\\"hRPaxjlTdR7wFMhV4w0b\\"}"' --form 'pronunciation_dictionary_locators="{\\"pronunciation_dictionary_id\\":\\"JzWtcGQMJ6bnlWwyMo7e\\",\\"version_id\\":\\"lbmwxiLu4q6txYxgdZqn\\"}"'.

        invalidate_affected_text : typing.Optional[bool]
            This will automatically mark text in this project for reconversion when the new dictionary applies or the old one no longer does.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreatePronunciationDictionaryResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs, PronunciationDictionaryVersionLocator

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.studio.projects.pronunciation_dictionaries.create(
            project_id="21m00Tcm4TlvDq8ikWAM",
            pronunciation_dictionary_locators=[
                PronunciationDictionaryVersionLocator(
                    pronunciation_dictionary_id="pronunciation_dictionary_id",
                )
            ],
        )
        """
        _response = self._raw_client.create(
            project_id,
            pronunciation_dictionary_locators=pronunciation_dictionary_locators,
            invalidate_affected_text=invalidate_affected_text,
            request_options=request_options,
        )
        return _response.data


class AsyncPronunciationDictionariesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPronunciationDictionariesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPronunciationDictionariesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPronunciationDictionariesClient
        """
        return self._raw_client

    async def create(
        self,
        project_id: str,
        *,
        pronunciation_dictionary_locators: typing.Sequence[PronunciationDictionaryVersionLocator],
        invalidate_affected_text: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreatePronunciationDictionaryResponseModel:
        """
        Create a set of pronunciation dictionaries acting on a project. This will automatically mark text within this project as requiring reconverting where the new dictionary would apply or the old one no longer does.

        Parameters
        ----------
        project_id : str
            The ID of the project to be used. You can use the [List projects](/docs/api-reference/studio/get-projects) endpoint to list all the available projects.

        pronunciation_dictionary_locators : typing.Sequence[PronunciationDictionaryVersionLocator]
            A list of pronunciation dictionary locators (pronunciation_dictionary_id, version_id) encoded as a list of JSON strings for pronunciation dictionaries to be applied to the text. A list of json encoded strings is required as adding projects may occur through formData as opposed to jsonBody. To specify multiple dictionaries use multiple --form lines in your curl, such as --form 'pronunciation_dictionary_locators="{\\"pronunciation_dictionary_id\\":\\"Vmd4Zor6fplcA7WrINey\\",\\"version_id\\":\\"hRPaxjlTdR7wFMhV4w0b\\"}"' --form 'pronunciation_dictionary_locators="{\\"pronunciation_dictionary_id\\":\\"JzWtcGQMJ6bnlWwyMo7e\\",\\"version_id\\":\\"lbmwxiLu4q6txYxgdZqn\\"}"'.

        invalidate_affected_text : typing.Optional[bool]
            This will automatically mark text in this project for reconversion when the new dictionary applies or the old one no longer does.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreatePronunciationDictionaryResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs, PronunciationDictionaryVersionLocator

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.studio.projects.pronunciation_dictionaries.create(
                project_id="21m00Tcm4TlvDq8ikWAM",
                pronunciation_dictionary_locators=[
                    PronunciationDictionaryVersionLocator(
                        pronunciation_dictionary_id="pronunciation_dictionary_id",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            project_id,
            pronunciation_dictionary_locators=pronunciation_dictionary_locators,
            invalidate_affected_text=invalidate_affected_text,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.get_knowledge_base_list_response_model import GetKnowledgeBaseListResponseModel
from ...types.get_or_create_rag_index_request_model import GetOrCreateRagIndexRequestModel
from ...types.knowledge_base_content_search_response_model import KnowledgeBaseContentSearchResponseModel
from ...types.knowledge_base_document_type import KnowledgeBaseDocumentType
from ...types.knowledge_base_sort_by import KnowledgeBaseSortBy
from ...types.sort_direction import SortDirection
from .raw_client import AsyncRawKnowledgeBaseClient, RawKnowledgeBaseClient
from .types.knowledge_base_get_or_create_rag_indexes_response_value import (
    KnowledgeBaseGetOrCreateRagIndexesResponseValue,
)

if typing.TYPE_CHECKING:
    from .document.client import AsyncDocumentClient, DocumentClient
    from .documents.client import AsyncDocumentsClient, DocumentsClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class KnowledgeBaseClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawKnowledgeBaseClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._documents: typing.Optional[DocumentsClient] = None
        self._document: typing.Optional[DocumentClient] = None

    @property
    def with_raw_response(self) -> RawKnowledgeBaseClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawKnowledgeBaseClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page_size: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        show_only_owned_documents: typing.Optional[bool] = None,
        created_by_user_id: typing.Optional[str] = None,
        types: typing.Optional[
            typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]
        ] = None,
        parent_folder_id: typing.Optional[str] = None,
        ancestor_folder_id: typing.Optional[str] = None,
        folders_first: typing.Optional[bool] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        sort_by: typing.Optional[KnowledgeBaseSortBy] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetKnowledgeBaseListResponseModel:
        """
        Get a list of available knowledge base documents

        Parameters
        ----------
        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100, defaults to 30.

        search : typing.Optional[str]
            If specified, the endpoint returns only such knowledge base documents whose names start with this string.

        show_only_owned_documents : typing.Optional[bool]
            If set to true, the endpoint will return only documents owned by you (and not shared from somebody else). Deprecated: use created_by_user_id instead.

        created_by_user_id : typing.Optional[str]
            Filter documents by creator user ID. When set, only documents created by this user are returned. Takes precedence over show_only_owned_documents. Use '@me' to refer to the authenticated user.

        types : typing.Optional[typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]]
            If present, the endpoint will return only documents of the given types.

        parent_folder_id : typing.Optional[str]
            If set, the endpoint will return only documents that are direct children of the given folder.

        ancestor_folder_id : typing.Optional[str]
            If set, the endpoint will return only documents that are descendants of the given folder.

        folders_first : typing.Optional[bool]
            Whether folders should be returned first in the list of documents.

        sort_direction : typing.Optional[SortDirection]
            The direction to sort the results

        sort_by : typing.Optional[KnowledgeBaseSortBy]
            The field to sort the results by

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetKnowledgeBaseListResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.list(
            page_size=1,
            search="search",
            show_only_owned_documents=True,
            created_by_user_id="created_by_user_id",
            types=["file"],
            parent_folder_id="parent_folder_id",
            ancestor_folder_id="ancestor_folder_id",
            folders_first=True,
            sort_direction="asc",
            sort_by="name",
            cursor="cursor",
        )
        """
        _response = self._raw_client.list(
            page_size=page_size,
            search=search,
            show_only_owned_documents=show_only_owned_documents,
            created_by_user_id=created_by_user_id,
            types=types,
            parent_folder_id=parent_folder_id,
            ancestor_folder_id=ancestor_folder_id,
            folders_first=folders_first,
            sort_direction=sort_direction,
            sort_by=sort_by,
            cursor=cursor,
            request_options=request_options,
        )
        return _response.data

    def get_or_create_rag_indexes(
        self,
        *,
        items: typing.Sequence[GetOrCreateRagIndexRequestModel],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, KnowledgeBaseGetOrCreateRagIndexesResponseValue]:
        """
        Retrieves and/or creates RAG indexes for multiple knowledge base documents in a single request. Maximum 100 items per request.

        Parameters
        ----------
        items : typing.Sequence[GetOrCreateRagIndexRequestModel]
            List of requested RAG indexes. Minimum 1, maximum 100 items.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, KnowledgeBaseGetOrCreateRagIndexesResponseValue]
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs, GetOrCreateRagIndexRequestModel

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.get_or_create_rag_indexes(
            items=[
                GetOrCreateRagIndexRequestModel(
                    document_id="document_id",
                    create_if_missing=True,
                    model="e5_mistral_7b_instruct",
                )
            ],
        )
        """
        _response = self._raw_client.get_or_create_rag_indexes(items=items, request_options=request_options)
        return _response.data

    def search(
        self,
        *,
        query: str,
        page_size: typing.Optional[int] = None,
        types: typing.Optional[
            typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]
        ] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> KnowledgeBaseContentSearchResponseModel:
        """
        Fuzzy text search over knowledge base document content

        Parameters
        ----------
        query : str
            The search query text

        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100, defaults to 30.

        types : typing.Optional[typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]]
            If present, the endpoint will return only documents of the given types.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        KnowledgeBaseContentSearchResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.knowledge_base.search(
            query="query",
            page_size=1,
            types=["file"],
            cursor="cursor",
        )
        """
        _response = self._raw_client.search(
            query=query, page_size=page_size, types=types, cursor=cursor, request_options=request_options
        )
        return _response.data

    @property
    def documents(self):
        if self._documents is None:
            from .documents.client import DocumentsClient  # noqa: E402

            self._documents = DocumentsClient(client_wrapper=self._client_wrapper)
        return self._documents

    @property
    def document(self):
        if self._document is None:
            from .document.client import DocumentClient  # noqa: E402

            self._document = DocumentClient(client_wrapper=self._client_wrapper)
        return self._document


class AsyncKnowledgeBaseClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawKnowledgeBaseClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._documents: typing.Optional[AsyncDocumentsClient] = None
        self._document: typing.Optional[AsyncDocumentClient] = None

    @property
    def with_raw_response(self) -> AsyncRawKnowledgeBaseClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawKnowledgeBaseClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page_size: typing.Optional[int] = None,
        search: typing.Optional[str] = None,
        show_only_owned_documents: typing.Optional[bool] = None,
        created_by_user_id: typing.Optional[str] = None,
        types: typing.Optional[
            typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]
        ] = None,
        parent_folder_id: typing.Optional[str] = None,
        ancestor_folder_id: typing.Optional[str] = None,
        folders_first: typing.Optional[bool] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        sort_by: typing.Optional[KnowledgeBaseSortBy] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetKnowledgeBaseListResponseModel:
        """
        Get a list of available knowledge base documents

        Parameters
        ----------
        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100, defaults to 30.

        search : typing.Optional[str]
            If specified, the endpoint returns only such knowledge base documents whose names start with this string.

        show_only_owned_documents : typing.Optional[bool]
            If set to true, the endpoint will return only documents owned by you (and not shared from somebody else). Deprecated: use created_by_user_id instead.

        created_by_user_id : typing.Optional[str]
            Filter documents by creator user ID. When set, only documents created by this user are returned. Takes precedence over show_only_owned_documents. Use '@me' to refer to the authenticated user.

        types : typing.Optional[typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]]
            If present, the endpoint will return only documents of the given types.

        parent_folder_id : typing.Optional[str]
            If set, the endpoint will return only documents that are direct children of the given folder.

        ancestor_folder_id : typing.Optional[str]
            If set, the endpoint will return only documents that are descendants of the given folder.

        folders_first : typing.Optional[bool]
            Whether folders should be returned first in the list of documents.

        sort_direction : typing.Optional[SortDirection]
            The direction to sort the results

        sort_by : typing.Optional[KnowledgeBaseSortBy]
            The field to sort the results by

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetKnowledgeBaseListResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.list(
                page_size=1,
                search="search",
                show_only_owned_documents=True,
                created_by_user_id="created_by_user_id",
                types=["file"],
                parent_folder_id="parent_folder_id",
                ancestor_folder_id="ancestor_folder_id",
                folders_first=True,
                sort_direction="asc",
                sort_by="name",
                cursor="cursor",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            page_size=page_size,
            search=search,
            show_only_owned_documents=show_only_owned_documents,
            created_by_user_id=created_by_user_id,
            types=types,
            parent_folder_id=parent_folder_id,
            ancestor_folder_id=ancestor_folder_id,
            folders_first=folders_first,
            sort_direction=sort_direction,
            sort_by=sort_by,
            cursor=cursor,
            request_options=request_options,
        )
        return _response.data

    async def get_or_create_rag_indexes(
        self,
        *,
        items: typing.Sequence[GetOrCreateRagIndexRequestModel],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Dict[str, KnowledgeBaseGetOrCreateRagIndexesResponseValue]:
        """
        Retrieves and/or creates RAG indexes for multiple knowledge base documents in a single request. Maximum 100 items per request.

        Parameters
        ----------
        items : typing.Sequence[GetOrCreateRagIndexRequestModel]
            List of requested RAG indexes. Minimum 1, maximum 100 items.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, KnowledgeBaseGetOrCreateRagIndexesResponseValue]
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs, GetOrCreateRagIndexRequestModel

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.get_or_create_rag_indexes(
                items=[
                    GetOrCreateRagIndexRequestModel(
                        document_id="document_id",
                        create_if_missing=True,
                        model="e5_mistral_7b_instruct",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_or_create_rag_indexes(items=items, request_options=request_options)
        return _response.data

    async def search(
        self,
        *,
        query: str,
        page_size: typing.Optional[int] = None,
        types: typing.Optional[
            typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]
        ] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> KnowledgeBaseContentSearchResponseModel:
        """
        Fuzzy text search over knowledge base document content

        Parameters
        ----------
        query : str
            The search query text

        page_size : typing.Optional[int]
            How many documents to return at maximum. Can not exceed 100, defaults to 30.

        types : typing.Optional[typing.Union[KnowledgeBaseDocumentType, typing.Sequence[KnowledgeBaseDocumentType]]]
            If present, the endpoint will return only documents of the given types.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        KnowledgeBaseContentSearchResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.knowledge_base.search(
                query="query",
                page_size=1,
                types=["file"],
                cursor="cursor",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.search(
            query=query, page_size=page_size, types=types, cursor=cursor, request_options=request_options
        )
        return _response.data

    @property
    def documents(self):
        if self._documents is None:
            from .documents.client import AsyncDocumentsClient  # noqa: E402

            self._documents = AsyncDocumentsClient(client_wrapper=self._client_wrapper)
        return self._documents

    @property
    def document(self):
        if self._document is None:
            from .document.client import AsyncDocumentClient  # noqa: E402

            self._document = AsyncDocumentClient(client_wrapper=self._client_wrapper)
        return self._document



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class VoiceStatisticsResponseModel(UncheckedBaseModel):
    project_voice_ref_id: str = pydantic.Field()
    """
    The project voice reference ID.
    """

    characters_unconverted: int = pydantic.Field()
    """
    The number of unconverted characters for this voice.
    """

    characters_converted: int = pydantic.Field()
    """
    The number of converted characters for this voice.
    """

    credits_needed_to_convert: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of credits needed to convert the remaining audio for this voice.
    """

    voice_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The voice ID.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
